module FlashFreeze_FSM #(
parameter [0:0] REG_FLASH_FREEZE = 1) 
(
input Flash_Freeze,     
clock, reset,          
safe_to_turn_off_clk,   
output reg housekeeping_request, 
output reg control_user_clock,   
output Flash_Freeze_Enabled
);

reg [2:0] ff_current_state /* synthesis syn_encoding="original" */;

parameter [2:0] wait_on_ff = 3'b000, persistent_ff = 3'b001,
house_keep_to_user_logic = 3'b010, stop_clock_to_user = 3'b011 , 
trigger_ulsicc_state = 3'b100, turn_on_user_clock = 3'b101,
fail_safe1 = 3'b110, fail_safe2 = 3'b111;

reg trigger_ulsicc /* synthesis syn_noprune = 1 */;

reg Flash_Freeze_reg, final_reg_unreg_ff;

assign Flash_Freeze_Enabled = trigger_ulsicc;

always@(Flash_Freeze or Flash_Freeze_reg)
begin
if(REG_FLASH_FREEZE)
final_reg_unreg_ff  <=  Flash_Freeze_reg;
else
final_reg_unreg_ff  <=  Flash_Freeze;
end

always@(posedge clock, negedge reset)
begin
if(reset == 1'b0)
begin
control_user_clock  <=  'b1;
trigger_ulsicc      <=  'b0;
housekeeping_request   <=  1'b0;
Flash_Freeze_reg       <=   1'b0;
ff_current_state       <=  wait_on_ff;
end

else
begin
Flash_Freeze_reg    <=  Flash_Freeze;
case(ff_current_state) /* synthesis parallel_case */

wait_on_ff:
    begin
    housekeeping_request   <=  1'b0;
    trigger_ulsicc         <=   1'b0;
    control_user_clock  <=  'b1;
    if(final_reg_unreg_ff  == 1'b0)
    ff_current_state   <=  persistent_ff;
    else
    ff_current_state   <=  wait_on_ff;
    end

persistent_ff:
    begin
    if(final_reg_unreg_ff  == 1'b0)
    ff_current_state   <=  house_keep_to_user_logic;
    else
    ff_current_state   <=  wait_on_ff;
    end

house_keep_to_user_logic:
    begin
    if(final_reg_unreg_ff  == 1'b0)
    begin
    housekeeping_request   <=  1'b1;
    ff_current_state   <=  stop_clock_to_user;
    end
    else
    begin
    ff_current_state   <=  wait_on_ff;
    end
    end

stop_clock_to_user:
    begin
    if(final_reg_unreg_ff  == 1'b0)
    begin
    if(safe_to_turn_off_clk == 1'b1)
    begin
    housekeeping_request   <=  1'b0;
    control_user_clock  <=  1'b0;
    ff_current_state       <=  trigger_ulsicc_state;    
    end
    else
    ff_current_state    <=  stop_clock_to_user;
    end
    else
    begin
    ff_current_state   <=  wait_on_ff;
    end
    end

trigger_ulsicc_state:
    begin
    if(final_reg_unreg_ff  == 1'b0)
    begin
    trigger_ulsicc   <=  1'b1;
    ff_current_state   <=  trigger_ulsicc_state;
    end
    else
    begin
    trigger_ulsicc   <=  1'b0;
    ff_current_state   <=  turn_on_user_clock;
    end
    end

turn_on_user_clock:
    begin
    ff_current_state   <=  wait_on_ff;
    end

fail_safe1:ff_current_state   <=  wait_on_ff;
fail_safe2:ff_current_state   <=  wait_on_ff;

default:
    begin
    ff_current_state   <=  wait_on_ff;
    end

endcase
end
end

ULSICC ULSICC_INSTANCE ( .LSICC(trigger_ulsicc) /* synthesis syn_noprune= 1 */);

endmodule
